@@ -1,211 +1,16 @@
-""" IPy - class and tools for handling of IPv4 and IPv6 Addresses and Networks.
+"""
+IPy - class and tools for handling of IPv4 and IPv6 addresses and networks.
+See README file for learn how to use IPy.
 
-Presentation of the API
-=======================
-
-The IP class allows a comfortable parsing and handling for most
-notations in use for IPv4 and IPv6 Addresses and Networks. It was
-greatly inspired bei RIPE's Perl module NET::IP's interface but
-doesn't share the Implementation. It doesn't share non-CIDR netmasks,
-so funky stuff lixe a netmask 0xffffff0f can't be done here.
-
-    >>> from IPy import IP
-    >>> ip = IP('127.0.0.0/30')
-    >>> for x in ip:
-    ...  print x
-    ...
-    127.0.0.0
-    127.0.0.1
-    127.0.0.2
-    127.0.0.3
-    >>> ip2 = IP('0x7f000000/30')
-    >>> ip == ip2
-    1
-    >>> ip.reverseNames()
-    ['0.0.0.127.in-addr.arpa.', '1.0.0.127.in-addr.arpa.', '2.0.0.127.in-addr.arpa.', '3.0.0.127.in-addr.arpa.']
-    >>> ip.reverseName()
-    '0-3.0.0.127.in-addr.arpa.'
-    >>> ip.iptype()
-    'PRIVATE'
-
-
-Support all IP addresses
-========================
-
-It can detect about a dozen different ways of expressing IP addresses
-and networks, parse them and distinguish between IPv4 and IPv6 addresses:
-
-    >>> IP('10.0.0.0/8').version()
-    4
-    >>> IP('::1').version()
-    6
-
-IPv4 addresses
---------------
-
-    >>> print IP(0x7f000001)
-    127.0.0.1
-    >>> print IP('0x7f000001')
-    127.0.0.1
-    >>> print IP('127.0.0.1')
-    127.0.0.1
-    >>> print IP('10')
-    10.0.0.0
-
-IPv6 addresses
---------------
-
-    >>> print IP('1080:0:0:0:8:800:200C:417A')
-    1080:0000:0000:0000:0008:0800:200c:417a
-    >>> print IP('1080::8:800:200C:417A')
-    1080:0000:0000:0000:0008:0800:200c:417a
-    >>> print IP('::1')
-    0000:0000:0000:0000:0000:0000:0000:0001
-    >>> print IP('::13.1.68.3')
-    0000:0000:0000:0000:0000:0000:0d01:4403
-
-Network mask
-------------
-
-    >>> print IP('127.0.0.0/8')
-    127.0.0.0/8
-    >>> print IP('127.0.0.0/255.0.0.0')
-    127.0.0.0/8
-    >>> print IP('127.0.0.0-127.255.255.255')
-    127.0.0.0/8
-
-
-Option check_addr_prefixlen
-===========================
-
-By default, IPy rejects uncommon netmask like 172.30.1.0/22:
-
-    >>> import IPy
-    >>> IPy.check_addr_prefixlen = True    # default value
-    >>> ips = IP('172.30.1.0/22')
-    Traceback (most recent call last):
-      ...
-    ValueError: IP('172.30.1.0/22') has invalid prefix length (22)
-
-You can change this behaviour with global option check_addr_prefixlen:
-
-    >>> IPy.check_addr_prefixlen = False   # disable
-    >>> ips = IP('172.30.1.0/22')
-    >>> len(ips)
-    1024
-
-
-Convert address to string
-=========================
-
-Nearly all class methods which return a string have an optional
-parameter 'wantprefixlen' which controlles if the prefixlen or netmask
-is printed. Per default the prefilen is always shown if the net
-contains more than one address::
-
-    wantprefixlen == 0 / None        don't return anything    1.2.3.0
-    wantprefixlen == 1               /prefix                  1.2.3.0/24
-    wantprefixlen == 2               /netmask                 1.2.3.0/255.255.255.0
-    wantprefixlen == 3               -lastip                  1.2.3.0-1.2.3.255
-
-You can also change the defaults on an per-object basis by fiddeling with the class members:
-
- * NoPrefixForSingleIp
- * WantPrefixLen
-
-Examples of string conversions:
-
-    >>> IP('10.0.0.0/32').strNormal()
-    '10.0.0.0'
-    >>> IP('10.0.0.0/24').strNormal()
-    '10.0.0.0/24'
-    >>> IP('10.0.0.0/24').strNormal(0)
-    '10.0.0.0'
-    >>> IP('10.0.0.0/24').strNormal(1)
-    '10.0.0.0/24'
-    >>> IP('10.0.0.0/24').strNormal(2)
-    '10.0.0.0/255.255.255.0'
-    >>> IP('10.0.0.0/24').strNormal(3)
-    '10.0.0.0-10.0.0.255'
-    >>> ip = IP('10.0.0.0')
-    >>> print ip
-    10.0.0.0
-    >>> ip.NoPrefixForSingleIp = None
-    >>> print ip
-    10.0.0.0/32
-    >>> ip.WantPrefixLen = 3
-    >>> print ip
-    10.0.0.0-10.0.0.0
-
-
-What's new?
-===========
-
-Changes between version 0.51 and 0.52:
-
- * Fix strCompressed() for IPv6 "ffff:ffff:ffff:ffff:ffff:f:f:fffc/127"
-
-Changes between version 0.5 and 0.51:
-
- * Use real name of IPy author
- * Use version "0.51" to help packaging since 0.5 was smaller than 0.42
- * Fix unit test for Python 2.3 (don't use doctest.testfile)
- * Fix unit test for Python 2.5 (problem of hex() lower case)
- * IPy now works on Python 2.2 to 2.5
-
-Changes between version 0.42 and 0.5: Fix all known bugs:
-
- * Apply Jean Gillaux's patch for netmask "/0.0.0.0" bug
- * Apply William McVey's patch for __nonzero__() bug
- * Apply Victor Stinner patch: setup.py can use setuptools and fix URLs
- * Allow "172.30.1.0/22" with new option IPy.check_addr_prefixlen=False
-
-Other changes:
-
- * Add regression tests
- * Create AUTHORS file
-
-
-Compatibility and links
-=======================
-
-IPy works on Python version 2.2 to 2.5.
-
-This Python module is under BSD license: see COPYING file.
-
-Further Information might be available at: http://software.inl.fr/trac/trac.cgi/wiki/IPy
-
-TODO
-====
-
- * better comparison (__cmp__ and friends)
- * tests for __cmp__
- * always write hex values lowercase
- * interpret 2001:1234:5678:1234/64 as 2001:1234:5678:1234::/64
- * move size in bits into class variables to get rid of some "if self._ipversion ..."
- * support for base85 encoding
- * support for output of IPv6 encoded IPv4 Addresses
- * update address type tables
- * first-last notation should be allowed for IPv6
- * add IPv6 docstring examples
- * check better for negative parameters
- * add addition / aggregation
- * move reverse name stuff out of the classes and refactor it
- * support for aggregation of more than two nets at once
- * support for aggregation with "holes"
- * support for finding common prefix
- * '>>' and '<<' for prefix manipulation
- * add our own exceptions instead ValueError all the time
- * rename checkPrefix to checkPrefixOk
- * add more documentation and doctests
- * refactor
+Further Information might be available at:
+http://software.inl.fr/trac/trac.cgi/wiki/IPy
 """
 
-# $HeadURL: https://svn.inl.fr/inl-svn/src/tools/ipy/tags/IPy-0.52/IPy.py $
-# $Id: IPy.py 5420 2006-11-06 09:27:13Z haypo $
+# $HeadURL: https://svn.inl.fr/inl-svn/src/tools/ipy/tags/IPy-0.55/IPy.py $
+# $Id: IPy.py 8947 2007-08-16 13:50:45Z haypo $
 
-__rcsid__ = '$Id: IPy.py 5420 2006-11-06 09:27:13Z haypo $'
-__version__ = '0.52'
+__rcsid__ = '$Id: IPy.py 8947 2007-08-16 13:50:45Z haypo $'
+__version__ = '0.55'
 
 import types
 
@@ -277,12 +82,12 @@ class IPint:
     Use class IP instead because some features are not implemented for
     IPint."""
 
-    def __init__(self, data, ipversion = 0):
+    def __init__(self, data, ipversion = 0, make_net = 0):
         """Create an instance of an IP object.
 
         Data can be a network specification or a single IP. IP
-        Addresses can be specified in all forms understood by
-        parseAddress.() the size of a network can be specified as
+        addresses can be specified in all forms understood by
+        parseAddress(). The size of a network can be specified as
 
         /prefixlen        a.b.c.0/24               2001:658:22a:cafe::/64
         -lastIP           a.b.c.0-a.b.c.255        2001:658:22a:cafe::-2001:658:22a:cafe:ffff:ffff:ffff:ffff
@@ -291,18 +96,28 @@ class IPint:
         If no size specification is given a size of 1 address (/32 for
         IPv4 and /128 for IPv6) is assumed.
 
+        If make_net is True, an IP address will be transformed into the network
+        address by applying the specified netmask.
+
         >>> print IP('127.0.0.0/8')
         127.0.0.0/8
         >>> print IP('127.0.0.0/255.0.0.0')
         127.0.0.0/8
         >>> print IP('127.0.0.0-127.255.255.255')
         127.0.0.0/8
+        >>> print IP('127.0.0.1/255.0.0.0', make_net=True)
+        127.0.0.0/8
 
         See module documentation for more examples.
         """
 
-        self.NoPrefixForSingleIp = 1  # Print no Prefixlen for /32 and /128
-        self.WantPrefixLen = None     # Do we want prefix printed by default? see _printPrefix()
+        global check_addr_prefixlen
+
+        # Print no Prefixlen for /32 and /128
+        self.NoPrefixForSingleIp = 1
+
+        # Do we want prefix printed by default? see _printPrefix()
+        self.WantPrefixLen = None
 
         netbits = 0
         prefixlen = -1
@@ -345,6 +160,13 @@ class IPint:
                     raise ValueError, "last address should be larger than first"
                 size = last - self.ip
                 netbits = _count1Bits(size)
+                if check_addr_prefixlen:
+                    # make sure the broadcast is the same as the last ip
+                    # otherwise it will return /16 for something like:
+                    # 192.168.0.0-192.168.191.255
+                    if IP('%s/%s' % (ip, 32-netbits)).broadcast().int() != last:
+                        raise ValueError, \
+                            "the range %s is not on a network boundary." % data
             elif len(x) == 1:
                 x = data.split('/')
                 # if no prefix is given use defaults
@@ -380,9 +202,12 @@ class IPint:
             self._ipversion = ipversion
             self._prefixlen = int(prefixlen)
 
-            if not _checkNetaddrWorksWithPrefixlen(self.ip, self._prefixlen, self._ipversion):
-                raise ValueError, "%s has invalid prefix length (%s)" % (repr(self), self._prefixlen)
+            if make_net:
+                self.ip = self.ip & _prefixlenToNetmask(self._prefixlen, self._ipversion)
 
+            if not _checkNetaddrWorksWithPrefixlen(self.ip,
+            self._prefixlen, self._ipversion):
+                raise ValueError, "%s has invalid prefix length (%s)" % (repr(self), self._prefixlen)
 
     def int(self):
         """Return the first / base / network addess as an (long) integer.
@@ -413,12 +238,14 @@ class IPint:
         return self._prefixlen
 
     def net(self):
-        """Return the base (first) address of a network as an (long) integer."""
-
+        """
+        Return the base (first) address of a network as an (long) integer.
+        """
         return self.int()
 
     def broadcast(self):
-        """Return the broadcast (last) address of a network as an (long) integer.
+        """
+        Return the broadcast (last) address of a network as an (long) integer.
 
         The same as IP[-1]."""
         return self.int() + self.len() - 1
@@ -427,7 +254,7 @@ class IPint:
         """Prints Prefixlen/Netmask.
 
         Not really. In fact it is our universal Netmask/Prefixlen printer.
-        This is considered an internel function.
+        This is considered an internal function.
 
         want == 0 / None        don't return anything    1.2.3.0
         want == 1               /prefix                  1.2.3.0/24
@@ -445,9 +272,10 @@ class IPint:
                 want = 1
         if want:
             if want == 2:
-                # this should work wit IP and IPint
+                # this should work with IP and IPint
                 netmask = self.netmask()
-                if type(netmask) != types.IntType and type(netmask) != types.LongType:
+                if type(netmask) != types.IntType \
+                and type(netmask) != types.LongType:
                     netmask = netmask.int()
                 return "/%s" % (intToIp(netmask, self._ipversion))
             elif want == 3:
@@ -458,7 +286,7 @@ class IPint:
         else:
             return ''
 
-        # We have different Favours to convert to:
+        # We have different flavours to convert to:
         # strFullsize   127.0.0.1    2001:0658:022a:cafe:0200:c0ff:fe8d:08fa
         # strNormal     127.0.0.1    2001:658:22a:cafe:200:c0ff:fe8d:08fa
         # strCompressed 127.0.0.1    2001:658:22a:cafe::1
@@ -505,7 +333,7 @@ class IPint:
             # find the longest sequence of '0'
             hextets = [int(x, 16) for x in self.strFullsize(0).split(':')]
             # every element of followingzeros will contain the number of zeros
-            # following the corrospondending element of hextetes
+            # following the corresponding element of hextets
             followingzeros = [0] * 8
             for i in range(len(hextets)):
                 followingzeros[i] = _countFollowingZeros(hextets[i:])
@@ -550,7 +378,7 @@ class IPint:
         return ret + self._printPrefix(wantprefixlen)
 
     def strFullsize(self, wantprefixlen = None):
-        """Return a string representation in the non mangled format.
+        """Return a string representation in the non-mangled format.
 
         >>> print IP('127.0.0.1').strFullsize()
         127.0.0.1
@@ -668,7 +496,7 @@ class IPint:
             raise ValueError, "only IPv4 and IPv6 supported"
 
     def len(self):
-        """Return the length of an subnet.
+        """Return the length of a subnet.
 
         >>> print IP('195.185.1.0/28').len()
         16
@@ -688,7 +516,7 @@ class IPint:
 
     def __nonzero__(self):
         """All IPy objects should evaluate to true in boolean context.
-        Ordinarily, they do, but if handling a default route expressed as
+        Ordinarily they do, but if handling a default route expressed as
         0.0.0.0/0, the __len__() of the object becomes 0, which is used
         as the boolean value of the object.
         """
@@ -696,13 +524,13 @@ class IPint:
 
 
     def __len__(self):
-        """Return the length of an subnet.
+        """Return the length of a subnet.
 
         Called to implement the built-in function len().
         It breaks with IPv6 Networks. Anybody knows how to fix this."""
 
         # Python < 2.2 has this silly restriction which breaks IPv6
-        # how about Python >= 2.2 ... ouch - it presists!
+        # how about Python >= 2.2 ... ouch - it persists!
 
         return int(self.len())
 
@@ -761,7 +589,7 @@ class IPint:
     def overlaps(self, item):
         """Check if two IP address ranges overlap.
 
-        Returns 0 if the two ranged don't overlap, 1 if the given
+        Returns 0 if the two ranges don't overlap, 1 if the given
         range overlaps at the end and -1 if it does at the beginning.
 
         >>> IP('192.168.0.0/23').overlaps('192.168.1.0/24')
@@ -795,7 +623,7 @@ class IPint:
         """Print a representation of the Object.
 
         Used to implement repr(IP). Returns a string which evaluates
-        to an identical Object (without the wnatprefixlen stuff - see
+        to an identical Object (without the wantprefixlen stuff - see
         module docstring.
 
         >>> print repr(IP('10.0.0.0/24'))
@@ -813,10 +641,10 @@ class IPint:
 
         Networks with different prefixlen are considered non-equal.
         Networks with the same prefixlen and differing addresses are
-        considered non equal but are compared by thair base address
+        considered non equal but are compared by their base address
         integer value to aid sorting of IP objects.
 
-        The Version of Objects is not put into consideration.
+        The version of Objects is not put into consideration.
 
         >>> IP('10.0.0.0/24') > IP('10.0.0.0')
         1
@@ -861,7 +689,7 @@ class IPint:
 
     def __hash__(self):
         """Called for the key object for dictionary operations, and by
-        the built-in function hash()  Should return a 32-bit integer
+        the built-in function hash(). Should return a 32-bit integer
         usable as a hash value for dictionary operations. The only
         required property is that objects which compare equal have the
         same hash value
@@ -880,7 +708,7 @@ class IPint:
 
 
 class IP(IPint):
-    """Class for handling IP Addresses and Networks."""
+    """Class for handling IP addresses and networks."""
 
     def net(self):
         """Return the base (first) address of a network as an IP object.
@@ -934,7 +762,7 @@ class IP(IPint):
         """
 
         if self._ipversion == 4:
-            ret =[]
+            ret = []
             # TODO: Refactor. Add support for IPint objects
             if self.len() < 2**8:
                 for x in self:
@@ -966,11 +794,11 @@ class IP(IPint):
 
 
     def reverseName(self):
-        """Return the value for reverse lookup/PTR records as RfC 2317 look alike.
+        """Return the value for reverse lookup/PTR records as RFC 2317 look alike.
 
-        RfC 2317 is an ugly hack which only works for sub-/24 e.g. not
-        for /23. Do not use it. Better set up a Zone for every
-        address. See reverseName for a way to arcive that.
+        RFC 2317 is an ugly hack which only works for sub-/24 e.g. not
+        for /23. Do not use it. Better set up a zone for every
+        address. See reverseName for a way to achieve that.
 
         >>> print IP('195.185.1.1').reverseName()
         1.1.185.195.in-addr.arpa.
@@ -1013,6 +841,19 @@ class IP(IPint):
         else:
             raise ValueError, "only IPv4 and IPv6 supported"
 
+    def make_net(self, netmask):
+        """Transform a single IP address into a network specification by
+        applying the given netmask.
+
+        Returns a new IP instance.
+
+        >>> print IP('127.0.0.1').make_net('255.0.0.0')
+        127.0.0.0/8
+        """
+        if '/' in str(netmask):
+            raise ValueError, "invalid netmask (%s)" % netmask
+        return IP('%s/%s' % (self, netmask), make_net=True)
+
     def __getitem__(self, key):
         """Called to implement evaluation of self[key].
 
@@ -1056,24 +897,156 @@ class IP(IPint):
             ret._prefixlen = self.prefixlen() - 1
             return ret
 
-def parseAddress(ipstr):
-    """Parse a string and return the corrospondending IPaddress and the a guess of the IP version.
 
-    Following Forms ar recorgnized:
-    0x0123456789abcdef           # IPv4 if <= 0xffffffff else IPv6
-    123.123.123.123              # IPv4
-    123.123                      # 0-padded IPv4
-    1080:0000:0000:0000:0008:0800:200C:417A
-    1080:0:0:0:8:800:200C:417A
-    1080:0::8:800:200C:417A
-    ::1
-    ::
-    0:0:0:0:0:FFFF:129.144.52.38
-    ::13.1.68.3
-    ::FFFF:129.144.52.38
+def _parseAddressIPv6(ipstr):
+    """
+    Internal function used by parseAddress() to parse IPv6 address with ':'.
+
+    >>> _parseAddressIPv6('::')
+    0L
+    >>> _parseAddressIPv6('::1')
+    1L
+    >>> _parseAddressIPv6('0:0:0:0:0:0:0:1')
+    1L
+    >>> _parseAddressIPv6('0:0:0::0:0:1')
+    1L
+    >>> _parseAddressIPv6('0:0:0:0:0:0:0:0')
+    0L
+    >>> _parseAddressIPv6('0:0:0::0:0:0')
+    0L
+
+    >>> _parseAddressIPv6('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210')
+    338770000845734292534325025077361652240L
+    >>> _parseAddressIPv6('1080:0000:0000:0000:0008:0800:200C:417A')
+    21932261930451111902915077091070067066L
+    >>> _parseAddressIPv6('1080:0:0:0:8:800:200C:417A')
+    21932261930451111902915077091070067066L
+    >>> _parseAddressIPv6('1080:0::8:800:200C:417A')
+    21932261930451111902915077091070067066L
+    >>> _parseAddressIPv6('1080::8:800:200C:417A')
+    21932261930451111902915077091070067066L
+    >>> _parseAddressIPv6('FF01:0:0:0:0:0:0:43')
+    338958331222012082418099330867817087043L
+    >>> _parseAddressIPv6('FF01:0:0::0:0:43')
+    338958331222012082418099330867817087043L
+    >>> _parseAddressIPv6('FF01::43')
+    338958331222012082418099330867817087043L
+    >>> _parseAddressIPv6('0:0:0:0:0:0:13.1.68.3')
+    218186755L
+    >>> _parseAddressIPv6('::13.1.68.3')
+    218186755L
+    >>> _parseAddressIPv6('0:0:0:0:0:FFFF:129.144.52.38')
+    281472855454758L
+    >>> _parseAddressIPv6('::FFFF:129.144.52.38')
+    281472855454758L
+    >>> _parseAddressIPv6('1080:0:0:0:8:800:200C:417A')
+    21932261930451111902915077091070067066L
+    >>> _parseAddressIPv6('1080::8:800:200C:417A')
+    21932261930451111902915077091070067066L
+    >>> _parseAddressIPv6('::1:2:3:4:5:6')
+    1208962713947218704138246L
+    >>> _parseAddressIPv6('1:2:3:4:5:6::')
+    5192455318486707404433266432802816L
+    """
+
+    # Split string into a list, example:
+    #   '1080:200C::417A' => ['1080', '200C', '417A'] and fill_pos=2
+    # and fill_pos is the position of '::' in the list
+    items = []
+    index = 0
+    fill_pos = None
+    while index < len(ipstr):
+        text = ipstr[index:]
+        if text.startswith("::"):
+            if fill_pos is not None:
+                # Invalid IPv6, eg. '1::2::'
+                raise ValueError("%r: Invalid IPv6 address: more than one '::'" % ipstr)
+            fill_pos = len(items)
+            index += 2
+            continue
+        pos = text.find(':')
+        if pos == 0:
+            # Invalid IPv6, eg. '1::2:'
+            raise ValueError("%r: Invalid IPv6 address" % ipstr)
+        if pos != -1:
+            items.append(text[:pos])
+            if text[pos:pos+2] == "::":
+                index += pos
+            else:
+                index += pos+1
+
+            if index == len(ipstr):
+                # Invalid IPv6, eg. '1::2:'
+                raise ValueError("%r: Invalid IPv6 address" % ipstr)
+        else:
+            items.append(text)
+            break
+
+    if items and '.' in items[-1]:
+        # IPv6 ending with IPv4 like '::ffff:192.168.0.1'
+        if not (fill_pos <= len(items)-1):
+            # Invalid IPv6: 'ffff:192.168.0.1::'
+            raise ValueError("%r: Invalid IPv6 address: '::' after IPv4" % ipstr)
+        value = parseAddress(items[-1])[0]
+        items = items[:-1] + ["%04x" % (value >> 16), "%04x" % (value & 0xffff)]
+
+    # Expand fill_pos to fill with '0'
+    # ['1','2'] with fill_pos=1 => ['1', '0', '0', '0', '0', '0', '0', '2']
+    if fill_pos is not None:
+        diff = 8 - len(items)
+        items = items[:fill_pos] + ['0']*diff + items[fill_pos:]
+
+    # Here we have a list of 8 strings
+    if len(items) != 8:
+        # Invalid IPv6, eg. '1:2:3'
+        raise ValueError("%r: Invalid IPv6 address: should have 8 hextets" % ipstr)
+
+    # Convert strings to long integer
+    value = 0L
+    index = 0
+    for item in items:
+        try:
+            item = int(item, 16)
+            error = not(0 <= item <= 0xFFFF)
+        except ValueError:
+            error = True
+        if error:
+            raise ValueError("%r: Invalid IPv6 address: invalid hexlet %r" % (ipstr, item))
+        value = (value << 16) + item
+        index += 1
+    return value
+
+def parseAddress(ipstr):
+    """
+    Parse a string and return the corresponding IP address (as integer)
+    and a guess of the IP version.
+
+    Following address formats are recognized:
+
+    >>> parseAddress('0x0123456789abcdef')           # IPv4 if <= 0xffffffff else IPv6
+    (81985529216486895L, 6)
+    >>> parseAddress('123.123.123.123')              # IPv4
+    (2071690107L, 4)
+    >>> parseAddress('123.123')                      # 0-padded IPv4
+    (2071658496L, 4)
+    >>> parseAddress('1080:0000:0000:0000:0008:0800:200C:417A')
+    (21932261930451111902915077091070067066L, 6)
+    >>> parseAddress('1080:0:0:0:8:800:200C:417A')
+    (21932261930451111902915077091070067066L, 6)
+    >>> parseAddress('1080:0::8:800:200C:417A')
+    (21932261930451111902915077091070067066L, 6)
+    >>> parseAddress('::1')
+    (1L, 6)
+    >>> parseAddress('::')
+    (0L, 6)
+    >>> parseAddress('0:0:0:0:0:FFFF:129.144.52.38')
+    (281472855454758L, 6)
+    >>> parseAddress('::13.1.68.3')
+    (218186755L, 6)
+    >>> parseAddress('::FFFF:129.144.52.38')
+    (281472855454758L, 6)
     """
 
-    # TODO: refactor me!
     if ipstr.startswith('0x'):
         ret = long(ipstr[2:], 16)
         if ret > 0xffffffffffffffffffffffffffffffffL:
@@ -1084,44 +1057,7 @@ def parseAddress(ipstr):
             return (ret, 6)
 
     if ipstr.find(':') != -1:
-        # assume IPv6
-        if ipstr.find(':::') != -1:
-            raise ValueError, "%r: IPv6 Address can't contain ':::'" % (ipstr)
-        hextets = ipstr.split(':')
-        if ipstr.find('.') != -1:
-            # this might be a mixed address like '0:0:0:0:0:0:13.1.68.3'
-            (v4, foo) = parseAddress(hextets[-1])
-            assert foo == 4
-            del(hextets[-1])
-            hextets.append(hex(v4 >> 16)[2:-1])
-            hextets.append(hex(v4 & 0xffff)[2:-1])
-        if len(hextets) > 8:
-            raise ValueError, "%r: IPv6 Address with more than 8 hexletts" % (ipstr)
-        if len(hextets) < 8:
-            if '' not in hextets:
-                raise ValueError, "%r IPv6 Address with less than 8 hexletts and without '::'" % (ipstr)
-            # catch :: at the beginning or end
-            if hextets.index('') < len(hextets) - 1 and hextets[hextets.index('')+1] == '':
-                hextets.remove('')
-            # catch '::'
-            if hextets.index('') < len(hextets) - 1 and hextets[hextets.index('')+1] == '':
-                hextets.remove('')
-
-            for foo in range(9-len(hextets)):
-                hextets.insert(hextets.index(''), '0')
-            hextets.remove('')
-            if '' in hextets:
-                raise ValueError, "%r IPv6 Address may contain '::' only once" % (ipstr)
-        if '' in hextets:
-            raise ValueError, "%r IPv6 Address may contain '::' only if it has less than 8 hextets" % (ipstr)
-        num = ''
-        for x in hextets:
-            if len(x) < 4:
-                x = ((4 - len(x)) * '0') + x
-            if int(x, 16) < 0 or int(x, 16) > 0xffff:
-                raise ValueError, "%r: single hextet must be 0 <= hextet <= 0xffff which isn't true for %s" % (ipstr, x)
-            num += x
-        return (long(num, 16), 6)
+        return (_parseAddressIPv6(ipstr), 6)
 
     elif len(ipstr) == 32:
         # assume IPv6 in pure hexadecimal notation
@@ -1143,9 +1079,9 @@ def parseAddress(ipstr):
         # we try to interprete it as a decimal digit -
         # this ony works for numbers > 255 ... others
         # will be interpreted as IPv4 first byte
-        ret = long(ipstr)
+        ret = long(ipstr, 10)
         if ret > 0xffffffffffffffffffffffffffffffffL:
-            raise ValueError, "IP Address cant be bigger than 2^128"
+            raise ValueError, "IP Address can't be bigger than 2^128"
         if ret <= 0xffffffffL:
             return (ret, 4)
         else:
@@ -1167,13 +1103,13 @@ def intToIp(ip, version):
             raise ValueError, "IPv4 Addresses can't be larger than 0xffffffff: %s" % (hex(ip))
         for l in range(4):
             ret = str(ip & 0xffL) + '.' + ret
-            ip = ip >> 8;
+            ip = ip >> 8
         ret = ret[:-1]
     elif version == 6:
         if ip > 0xffffffffffffffffffffffffffffffffL:
             raise ValueError, "IPv6 Addresses can't be larger than 0xffffffffffffffffffffffffffffffff: %s" % (hex(ip))
         l = '0' * 32 + hex(ip)[2:-1]
-        for x in range(1,33):
+        for x in range(1, 33):
             ret = l[-x] + ret
             if x % 4 == 0:
                 ret = ':' + ret
@@ -1181,7 +1117,7 @@ def intToIp(ip, version):
     else:
         raise ValueError, "only IPv4 and IPv6 supported"
 
-    return ret;
+    return ret
 
 def _ipVersionToLen(version):
     """Return number of bits in address for a certain IP version.
@@ -1207,7 +1143,7 @@ def _ipVersionToLen(version):
 
 
 def _countFollowingZeros(l):
-    """Return Nr. of elements containing 0 at the beginning th the list."""
+    """Return number of elements containing 0 at the beginning of the list."""
     if len(l) == 0:
         return 0
     elif l[0] != 0:
@@ -1225,7 +1161,7 @@ def _intToBin(val):
     """Return the binary representation of an integer as string."""
 
     if val < 0:
-        raise ValueError, "Only positive Values allowed"
+        raise ValueError, "Only positive values allowed"
     s = hex(val).lower()
     ret = ''
     if s[-1] == 'l':
@@ -1297,7 +1233,7 @@ def _checkPrefix(ip, prefixlen, version):
 
 
 def _checkNetmask(netmask, masklen):
-    """Checks if a netmask is expressable as e prefixlen."""
+    """Checks if a netmask is expressable as a prefixlen."""
 
     num = long(netmask)
     bits = masklen
@@ -1317,7 +1253,7 @@ def _checkNetmask(netmask, masklen):
 
 
 def _checkNetaddrWorksWithPrefixlen(net, prefixlen, version):
-    """Check if a base addess of e network is compatible with a prefixlen"""
+    """Check if a base addess of a network is compatible with a prefixlen"""
     global check_addr_prefixlen
     if check_addr_prefixlen:
         if net & _prefixlenToNetmask(prefixlen, version) == net:
@@ -1333,7 +1269,7 @@ def _checkNetaddrWorksWithPrefixlen(net, prefixlen, version):
 
 
 def _netmaskToPrefixlen(netmask):
-    """Convert an Integer reprsenting a Netmask to an prefixlen.
+    """Convert an Integer representing a netmask to a prefixlen.
 
     E.g. 0xffffff00 (255.255.255.0) returns 24
     """
@@ -1347,7 +1283,8 @@ def _netmaskToPrefixlen(netmask):
 def _prefixlenToNetmask(prefixlen, version):
     """Return a mask of n bits as a long integer.
 
-    From 'IP address conversion functions with the builtin socket module' by Alex Martelli
+    From 'IP address conversion functions with the builtin socket module'
+    by Alex Martelli
     http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66517
     """
     if prefixlen == 0:
@@ -1357,15 +1294,10 @@ def _prefixlenToNetmask(prefixlen, version):
     return ((2L<<prefixlen-1)-1) << (_ipVersionToLen(version) - prefixlen)
 
 
-def _test():
-    import doctest, IPy
-    return doctest.testmod(IPy)
-
 if __name__ == "__main__":
-    _test()
+    import doctest
+    failure, nbtest = doctest.testmod()
+    if failure:
+        import sys
+        sys.exit(1)
 
-    t = [0xf0, 0xf00, 0xff00, 0xffff00, 0xffffff00L]
-    o = []
-    for x in t:
-        pass
-    x = 0L
